#include "ThreadUtil.h"
#include <windows.h>
#include <process.h>
#include <queue>
#include <mutex>
#include <condition_variable>
#include <memory>

namespace ThreadUtil {
    // Global variables for thread management
    static DWORD g_platformThreadId = 0;
    static std::queue<std::function<void()>> g_taskQueue;
    static std::mutex g_queueMutex;
    static std::condition_variable g_queueCondition;
    static bool g_initialized = false;
    static HANDLE g_workerThread = NULL;
    // Worker thread function
    unsigned __stdcall WorkerThreadProc(void* param) {
        while (true) {
            std::function<void()> task;

            {
                std::unique_lock<std::mutex> lock(g_queueMutex);
                g_queueCondition.wait(lock, [] { return !g_taskQueue.empty(); });
                task = g_taskQueue.front();
                g_taskQueue.pop();
            }

            // Execute the task
            task();
        }

        return 0;
    }
    // Function to initialize the platform thread ID
    void InitializePlatformThreadId() {
        g_platformThreadId = GetCurrentThreadId();
        
        // Create worker thread
        g_workerThread = (HANDLE)_beginthreadex(NULL, 0, WorkerThreadProc, NULL, 0, NULL);
        g_initialized = true;
    }

   

    // Function to ensure code runs on the platform thread
    void RunOnPlatformThread(std::function<void()> callback) {
        // Initialize on first call
        if (!g_initialized) {
            InitializePlatformThreadId();


        }

        // If we're already on the platform thread, execute directly
        if (GetCurrentThreadId() == g_platformThreadId) {
            callback();
            return;
        }

        // Create a shared_ptr to the callback to ensure it's copy constructible
        auto callbackPtr = std::make_shared<std::function<void()>>(std::move(callback));

        // Otherwise, queue the task for execution on the platform thread
        {
            std::lock_guard<std::mutex> lock(g_queueMutex);
            g_taskQueue.push([callbackPtr]() { (*callbackPtr)(); });
        }
        g_queueCondition.notify_one();
    }

    // Helper function to invoke method on platform thread
    void InvokeMethodOnPlatformThread(
        flutter::MethodChannel<flutter::EncodableValue>* channel,
        const std::string& method,
        std::unique_ptr<flutter::EncodableValue> arguments) {

        if (!channel) return;

        // Create shared pointers for the data to ensure copy constructibility
        auto methodCopy = std::make_shared<std::string>(method);
        auto argsCopy = std::shared_ptr<flutter::EncodableValue>(arguments.release());

        RunOnPlatformThread([channel, methodCopy, argsCopy]() {
            channel->InvokeMethod(
                *methodCopy,
                std::make_unique<flutter::EncodableValue>(*argsCopy)
            );
        });
    }
}
